package com.tofly.common.log.aspect;

import com.tofly.base.api.entity.SysLog;
import com.tofly.common.core.util.SpringContextHolder;
import com.tofly.common.log.annotation.Log;
import com.tofly.common.log.event.SysLogEvent;
import com.tofly.common.log.util.SysLogUtils;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.Signature;
import org.aspectj.lang.annotation.*;
import org.aspectj.lang.reflect.MethodSignature;

import java.lang.reflect.Method;

/**
 * @author Administrator
 * @date 2019/8/30  17:23
 */

@Aspect
@Slf4j
public class LogAspect {
	// 配置织入点
	@Pointcut("@annotation(com.tofly.common.log.annotation.Log)")
	public void logPointCut() {
	}
	
	@Before(value="logPointCut()")
	public void beforeLog(JoinPoint joinPoint) {
		//handleLog(joinPoint);
	}
	
	@After(value="logPointCut()")
	public void afterLog(JoinPoint joinPoint) {
		//handleLog(joinPoint);
	}
	
	@Around(value="logPointCut()")
	@SneakyThrows
	public Object round(ProceedingJoinPoint joinPoint) {
		return handleLog(joinPoint);
	}


	public Object handleLog(final ProceedingJoinPoint joinPoint) throws Exception{
		try {
			Log controllerLog = getAnnotationLog(joinPoint);
			if(controllerLog==null) {
				return null;
			}
			/*HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
			System.out.println("IP:"+request.getRemoteAddr());*/
			long startTime = System.currentTimeMillis();//开始时间

	        Object[] args = joinPoint.getArgs();//获取参数数组
	        Object o=null;
	        try {
				o = joinPoint.proceed(args);
			} catch (Throwable e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				throw new Exception(e.getMessage());
			}
	        long endTime = System.currentTimeMillis();//结束时间
			String logg="title:"+controllerLog.title()+","+
					"businessType"+controllerLog.businessType()+","+
					"operatorType"+controllerLog.operatorType()+","+
					"isSaveRequestData"+controllerLog.isSaveRequestData();
			// 设置方法名称
			String className = joinPoint.getTarget().getClass().getName();
			String methodName = joinPoint.getSignature().getName();
			SysLog sysLog= SysLogUtils.getSysLog();
			sysLog.setTitl(logg);
			sysLog.setTime(endTime-startTime);

			SpringContextHolder.publishEvent(new SysLogEvent(sysLog));

			log.info(logg);
			return o;
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return null;
		}
	}
	
	//判断是否存在注解
	private Log getAnnotationLog(JoinPoint joinPoint) throws Exception {
		
		
		Signature signature = joinPoint.getSignature();
		MethodSignature methodSignature = (MethodSignature) signature;
		Method method = methodSignature.getMethod();
		int i=0;
		for( String s :methodSignature.getParameterNames()) {
			System.out.println(s+":"+joinPoint.getArgs()[i]);
			i++;
		}
		
		if (method != null) {
			return method.getAnnotation(Log.class);
		}
		return null;
	}
}
