package com.tofly.common.oauth.auth;

import com.tofly.base.api.dto.UserInfo;
import com.tofly.base.api.entity.SysUser;
import com.tofly.base.api.feign.BaseUserService;
import com.tofly.common.core.constant.CommonConstants;
import com.tofly.common.core.constant.SecurityConstants;
import com.tofly.common.core.entity.ResultRespone;
import lombok.AllArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.cache.Cache;
import org.springframework.cache.CacheManager;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.AuthorityUtils;
import org.springframework.security.core.userdetails.User;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

/**
 * @author Administrator
 * @date 2019/9/6  14:48
 * 自定义用户认证
 */
@Service
@Slf4j
@AllArgsConstructor
public class ToFlyUserDetailsService implements UserDetailsService {

    private final CacheManager cacheManager;
    private final BaseUserService baseUserService;

    @Override
    @SneakyThrows
    public UserDetails loadUserByUsername(String username){
        //首先从缓存中获取，当调用时，我们可以将用户信息存储到缓存，或者程序启动时加载到缓存
        Cache cache = cacheManager.getCache("user_details");
        if (cache != null && cache.get(username) != null) {
            return (User) cache.get(username).get();
        }
        log.info("------------baseUserService:{},username:{}",baseUserService,username);
        ResultRespone<UserInfo> result = baseUserService.getInfo(username, SecurityConstants.FROM_IN);
        UserDetails userDetails=getUserDetails(result);
        //放入缓存
        cache.put(username,userDetails);

        return userDetails;
    }

    /**
     * 将用户设置为对应的oauth形式
     * @param result
     * @return
     */
    private UserDetails getUserDetails(ResultRespone<UserInfo> result) {
        if (result == null || result.getData() == null) {
            throw new UsernameNotFoundException(result+"用户不存在");
        }
        UserInfo userInfo=result.getData();
        Set<String> dbauthSet=new HashSet<>();
        Integer[] roles=userInfo.getRoles();
        if(roles!=null && roles.length>0){
            //添加角色
            Arrays.stream(roles).forEach(role->dbauthSet.add(SecurityConstants.ROLE+role));
            //添加权限
            dbauthSet.addAll(Arrays.asList(userInfo.getPermissions()));
        }

        SysUser user=userInfo.getSysUser();
        Collection<? extends GrantedAuthority> authorities
                = AuthorityUtils.createAuthorityList(dbauthSet.toArray(new String[0]));

        return new User(user.getUsername(),/*SecurityConstants.BCRYPT+*/user.getPassword(),
                CommonConstants.STATUS_NORMAL.equals(user.getLockFlag()),true,
                true,true,authorities);
    }
}
